/* Code for Bellemare, Marc F., and		*/
/* Christopher B. Barrett, 2006, 		*/
/* "An Ordered Tobit Model of Market	*/
/* Participation," American Journal of	*/
/* Agricultural Economics.			*/

/*							*/
/* Code Author: Marc F. Bellemare		*/
/* Ph.D. Candidate				*/
/* Cornell University				*/
/* mfb24@cornell.edu				*/


clear
drop _all
set mem 640m
set matsize 3000
set more off
version 8

use "C:\BellemareBarrettOrderedTobit.dta"


/* Data Management				*/

drop if code==24011
drop if code==13281
drop if code==11211
drop if code==15091
drop if code==12251
drop if code==16161
drop if code==16181

gen lnpls=log(avgpls)
gen lnpss=log(avgpss)

gen mar = 1 if _301==1 | _302==1
replace mar = 0 if mar==.

gen jun = 1 if _600==1 | _601==1 | _602==1
replace jun = 0 if jun==.

gen sep = 1 if _900==1 | _901==1
replace sep = 0 if sep==.

gen dec = 1 if _1200==1 | _1201==1
replace dec = 0 if dec==.

drop depr
gen depr = (hunder15 + hover64)/hhsize

gen kargi1 = _900*kargi
gen logo1 = _900*logo
gen ngambo1 = _900*ngambo
gen nhorr1 = _900*nhorr
gen sm1 = _900*sm
gen dida1 = _900*dida 
gen dillo1 = _900*dillo
gen qorate1 = _900*qorate

gen kargi2 = _1200*kargi
gen logo2 = _1200*logo
gen ngambo2 = _1200*ngambo
gen nhorr2 = _1200*nhorr
gen sm2 = _1200*sm
gen dida2 = _1200*dida 
gen dillo2 = _1200*dillo
gen qorate2 = _1200*qorate

gen kargi3 = _301*kargi
gen logo3 = _301*logo
gen ngambo3 = _301*ngambo
gen nhorr3 = _301*nhorr
gen sm3 = _301*sm
gen dida3 = _301*dida 
gen dillo3 = _301*dillo
gen qorate3 = _301*qorate

gen kargi4 = _601*kargi
gen logo4 = _601*logo
gen ngambo4 = _601*ngambo
gen nhorr4 = _601*nhorr
gen sm4 = _601*sm
gen dida4 = _601*dida 
gen dillo4 = _601*dillo
gen qorate4 = _601*qorate

gen kargi5 = _901*kargi
gen logo5 = _901*logo
gen ngambo5 = _901*ngambo
gen nhorr5 = _901*nhorr
gen sm5 = _901*sm
gen dida5 = _901*dida 
gen dillo5 = _901*dillo
gen qorate5 = _901*qorate

gen kargi6 = _1201*kargi
gen logo6 = _1201*logo
gen ngambo6 = _1201*ngambo
gen nhorr6 = _1201*nhorr
gen sm6 = _1201*sm
gen dida6 = _1201*dida 
gen dillo6 = _1201*dillo
gen qorate6 = _1201*qorate

gen kargi7 = _302*kargi
gen logo7 = _302*logo
gen ngambo7 = _302*ngambo
gen nhorr7 = _302*nhorr
gen sm7 = _302*sm
gen dida7 = _302*dida 
gen dillo7 = _302*dillo
 gen qorate7 = _302*qorate

gen kargi8 = _602*kargi
gen logo8 = _602*logo
gen ngambo8 = _602*ngambo
gen nhorr8 = _602*nhorr
gen sm8 = _602*sm
gen dida8 = _602*dida 
gen dillo8 = _602*dillo
gen qorate8 = _602*qorate


log using "C:\BellemareBarrettOrderedTobit", replace


/* Instrumentation of Income			*/
/* with Education Dummies			*/


reg total_inc kargi1 logo1 ngambo1 nhorr1 sm1 dida1 dillo1 qorate1 kargi2 logo2 ngambo2 nhorr2 sm2 dida2 dillo2 qorate2 kargi3 logo3 ngambo3 nhorr3 sm3 dida3 dillo3 qorate3 kargi4 logo4 ngambo4 nhorr4 sm4 dida4 dillo4 qorate4 kargi5 logo5 ngambo5 nhorr5 sm5 dida5 dillo5 qorate5 kargi6 logo6 ngambo6 nhorr6 sm6 dida6 dillo6 qorate6 kargi7 logo7 ngambo7 nhorr7 sm7 dida7 dillo7 qorate7 kargi8 logo8 ngambo8 nhorr8 sm8 dida8 dillo8 qorate8 tot_tlu heduc112 heduc13 heduca landtot
predict yhat
gen instr = yhat
drop yhat


/* Inverted Mills Ratios			*/

oprobit mktpart jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hunder15 hhsize births deaths hheadsex hheadage hheadage2 landtot assets instr depr tot_tlu pcttlufe malencum femencum fixfees fixfees2 tlubt
predict lp, xb
gen IMR_b = normden(-1.082337-lp)/norm(-1.082337-lp)
gen IMR_s = normden(lp-1.399361)/norm(lp-1.399361)

/* Maximum Likelihood Program			*/
/*  for the "Ordered Tobit"			*/

gen logLOT = .

program mktpart_lf
	version 8.2
	args lnf xb1 xb2 xb3 alpha1 alpha2 sigma2 sigma3
	quietly replace `lnf' = ln(norm((`alpha1' - `xb1'))) + ln(normden($ML_y2, `xb2', `sigma2')) if mktpart == 0
	quietly replace `lnf' = ln(norm((`alpha2' - `xb1')) - norm((`alpha1' - `xb1'))) if mktpart == 1
	quietly replace `lnf' = ln(1 - norm((`alpha2' -`xb1'))) + ln(normden($ML_y3, `xb3', `sigma3')) if mktpart == 2
	quietly replace logLOT = `lnf'

end

ml model lf mktpart_lf (MktPart: mktpart = jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hunder15 hhsize births deaths hheadsex hheadage hheadage2 landtot assets instr depr tot_tlu pcttlufe malencum femencum fixfees fixfees2 tlubt, noconstant) (Bought: netpurch =  deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate IMR_b hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees) (Sold: netsales = death births jun sep dec kargi logo ngambo nhorr sm  dida dillo qorate IMR_s hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees) (Alpha1: ) (Alpha2: ) (Sigma2:) (Sigma3:), title(Ordered Tobit for Market Participation)

ml check

ml maximize, difficult

predict yhat1, eq(MktPart)
predict yhat2, eq(Bought)
predict yhat3, eq(Sold)
predict yhat4, eq(Sigma2)
predict yhat5, eq(Sigma3)
gen e1 = mktpart - yhat1
gen e2 = netsales - yhat2
gen e3 = netsales - yhat3
rename yhat1 mu1
rename yhat2 mu2
rename yhat3 mu3
rename yhat4 sig2
rename yhat5 sig3

pwcorr e1 e2 e3, star(.01)

pwcorr e1 e2 e3, star(.05)

pwcorr e1 e2 e3, star(.01)


/* Define Greene's (2003)			*/
/* AsyVar(gamma_hat)				*/

oprobit mktpart jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hunder15 hhsize births deaths hheadsex hheadage hheadage2 landtot assets instr depr tot_tlu pcttlufe malencum femencum fixfees fixfees2 tlubt

matrix V = get(VCE)

drop kargi1 logo1 ngambo1 nhorr1 sm1 dida1 dillo1 kargi2 qorate1 logo2 ngambo2 nhorr2 sm2 dida2 dillo2 qorate2 kargi3 logo3 ngambo3 nhorr3 sm3 dida3 dillo3 qorate3 kargi4 logo4 ngambo4 nhorr4 sm4 dida4 dillo4 qorate4 kargi5 logo5 ngambo5 nhorr5 sm5 dida5 dillo5 qorate5 kargi6 logo6 ngambo6 nhorr6 sm6 dida6 dillo6 qorate6 kargi7 logo7 ngambo7 nhorr7 sm7 dida7 dillo7 qorate7 kargi8 logo8 ngambo8 nhorr8 sm8 dida8 dillo8 qorate8

drop if e2==.

/* Defines Greene's (2003) "delta hat"	*/
/* and "lambda hat"				*/

gen d_hat_1b = IMR_b*(IMR_b + 1.082347 - lp)
gen d_hat_1s = IMR_s*(IMR_s - 1.399361 - lp)


/* Defines Greene's (2003) "delta bar"	*/

gen d_bar_1b = 5.751259
gen d_bar_1s = -.9931579


/* Defines Greene's (2003) "sigmas"		*/

mkmat e2
matrix e2p = e2'
mkmat e3
matrix e3p = e3'
mkmat e1

matrix s_hat_1b = (1/2037)*(e2p*e2) + 5.751259*(19.86553)^2
matrix s_hat_1s = (1/2037)*(e3p*e3) - 0.9931579*(-1.277819)^2

matrix list s_hat_1b
matrix list s_hat_1s


/* Defines Greene's (2003) "rhos"		*/

matrix rho_h_1b =  (19.86553)^2*inv(s_hat_1b)
matrix rho_h_1s =  (-1.277819)^2*inv(s_hat_1s)

matrix list rho_h_1b
matrix list rho_h_1s


/* NOTE: These are just the diagonal	*/
/* terms, NOT a matrix.				*/
/* NOTE: Rho does NOT have to be in 	*/
/* the [0,1] interval! This is OK.		*/

gen rho_h_1b = .16576579
gen rho_h_1s = 2.2817722

gen diagb = 1 - rho_h_1b*d_hat_1b
gen diags = 1 - rho_h_1s*d_hat_1s

mkmat diagb
mkmat diags

mkmat d_hat_1b
mkmat d_hat_1s

matrix drop e1 e2 e2p e3 e3p

matrix Cb = diag(diagb)
matrix Cs = diag(diags)

matrix Db = diag(d_hat_1b)
matrix Ds = diag(d_hat_1s)


/* Data Matrices					*/

gen const = 1

mkmat const

mkmat jun sep dec kargi logo ngambo nhorr sm  dida dillo qorate deaths births hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees IMR_b const, matrix(Xb)

mkmat jun sep dec kargi logo ngambo nhorr sm  dida dillo qorate deaths births hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees IMR_s const, matrix(Xs)

mkmat jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hunder15 hhsize births deaths hheadsex hheadage hheadage2 landtot assets instr depr tot_tlu pcttlufe malencum femencum fixfees fixfees2 tlubt, matrix(W)


/* Q Matrix						*/


matrix Y = V["jun".."tlubt","jun".."tlubt"]

matrix Qb = rho_h_1b*(Xb'*Db*W)*Y*(W'*Db*Xb)
matrix Qs = rho_h_1s*(Xs'*Db*W)*Y*(W'*Db*Xs)



/* Final Matrices					*/

matrix VCb = s_hat_1b*inv(Xb'*Xb)*(Xb'*Cb*Xb + Qb)*inv(Xb'*Xb)
matrix VCs = s_hat_1s*inv(Xs'*Xs)*(Xs'*Cs*Xs + Qs)*inv(Xs'*Xs)

matrix list VCb
matrix list VCs


/* Biavariate Tobit Model			*/


/* Data Management				*/

gen d = 1 if autarkic==1
replace d = 0 if buyer==1
replace d = 2 if seller==1
gen sales = netsales if netsales>0
replace sales = 0 if netsales<=0
gen purch = netpurch if netpurch>0
replace purch = 0 if netpurch<=0
gen logLBT = .


/* Maximum Likelihood Program			*/
/*  for a Bivariate Tobit			*/

program mybtobit_lf
	args lnf xb1 xb2 sigma1 sigma2 rho

	quietly replace `lnf' = ln(normden($ML_y1,`xb1',`sigma1')*(1-norm(($ML_y2-`xb2'-(2/_pi*atan(`rho'))*`sigma2'/`sigma1'*($ML_y1-`xb1'))/(`sigma2'*sqrt(1-(2/_pi*atan(`rho'))^2))))) if d == 0
	quietly replace `lnf' = ln(normden($ML_y2,`xb2',`sigma2')*(1-norm(($ML_y1-`xb1'-(2/_pi*atan(`rho'))*`sigma1'/`sigma2'*($ML_y2-`xb2'))/(`sigma1'*sqrt(1-(2/_pi*atan(`rho'))^2))))) if d == 2
	quietly replace `lnf' = ln(1-binorm(($ML_y1-`xb1')/`sigma1',($ML_y2-`xb2')/`sigma2',(2/_pi*atan(`rho')))) if d == 1

	quietly replace logLBT = `lnf'
end

ml model lf mybtobit_lf (Buyers: purch = deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees) (Sellers: sales = deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees) (Sigma1: ) (Sigma2: ) (Rho: )

ml check

ml maximize, difficult

predict yhat1, eq(Buyers)
predict yhat2, eq(Sellers)

rename yhat1 mubt1
rename yhat2 mubt2

gen corr = 2/_pi*atan(-1.345539)

sum corr

rename logLBT logLB
rename logLOT logLO
gen logLBT=.
gen logLOT=.


ml model lf mybtobit_lf (Buyers: purch = deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees mu2) (Sellers: sales = deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees mu3) (Sigma1: ) (Sigma2: ) (Rho: ), title(Bivariate Tobit: J-Test)

ml check

ml maximize, difficult

test [Buyers]mu2=0
test [Sellers]mu3=0, accumulate

ml model lf mktpart_lf (MktPart: mktpart = jun sep dec kargi logo ngambo nhorr sm dida dillo qorate hunder15 hhsize births deaths hheadsex hheadage hheadage2 landtot assets instr depr tot_tlu pcttlufe malencum femencum fixfees fixfees2 tlubt, noconstant) (Bought: netpurch =  deaths births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate IMR_b hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees mubt1) (Sold: netsales = death births jun sep dec kargi logo ngambo nhorr sm dida dillo qorate IMR_s hheadsex hheadage hheadage2 hhsize landtot assets instr depr tot_tlu pcttlufe malencum femencum lnpls lnpss fixfees varfees mubt2) (Alpha1: ) (Alpha2: ) (Sigma2:) (Sigma3:), title(Ordered Tobit: J-Test) technique(dfp)

ml maximize, difficult

test [Bought]mubt1=0
test [Sold]mubt2=0, accumulate

gen sales1 = netsales if netsales>0
replace sales1 = . if netsales==0

gen purch1 = netpurch if netpurch>0
replace purch1 = . if netpurch==0

gen buy1 = 1 if d==0
replace buy1 = 0 if d~=0
gen auta1 = 1 if d==1
replace auta1 = 0 if d~=1
gen sell1 = 1 if d==2
replace sell1 = 0 if d~=2

sort d

by d: sum hheadsex hheadage depr hhsize landtot assets births deaths income tot_tlu pcttlufe malencum femencum avgpls avgpss tlubt buyer seller autarkic netsales fixfees varfees sales1 purch1 buy1 auta1 sell1

sum hheadsex hheadage depr hhsize landtot assets births deaths income tot_tlu pcttlufe malencum femencum avgpls avgpss tlubt buyer seller autarkic netsales fixfees varfees sales1 purch1 buy1 auta1 sell1


log close






